<!--
SPDX-FileCopyrightText: {{ CONFIG.authors }}

SPDX-License-Identifier: {{ CONFIG.license }}
-->

<!DOCTYPE html>
<html lang="en">
<head>
  <title>{% block title %}{{ namespace.name }}{% endblock %}</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <meta charset="utf-8" />

  {% if CONFIG.site_url %}
  <meta property="og:url" content="{{ CONFIG.site_url }}"/>
  {% endif %}
  <meta property="og:type" content="website"/>

  {% if CONFIG.logo_url %}
  <meta property="og:image:width" content="256"/>
  <meta property="og:image:height" content="256"/>
  <meta property="og:image:secure_url" content="{{ CONFIG.logo_url }}"/>
  <meta property="og:image:alt" content="{{ namespace.name }}-{{ namespace.version }}"/>
  {% endif %}

  {% block meta_other %}{% endblock %}

  {% if CONFIG.site_url %}
  <meta name="twitter:url" content="{{ CONFIG.site_url }}"/>
  {% endif %}
  <meta name="twitter:card" content="summary"/>

  {% if CONFIG.site_url %}
  <link rel="canonical" href="{{ CONFIG.site_url }}"/>
  {% endif %}
  {% if CONFIG.search_index %}
  {% if CONFIG.docs_url %}
  <link rel="search" type="application/opensearchdescription+xml" title="{{ namespace.name }}" href="opensearch.xml">
  {% endif %}
  {% endif %}

  <link rel="stylesheet" href="style.css" type="text/css" />

  {% block style_other %}{% endblock %}

  {% if CONFIG.urlmap_file %}
  <script src="{{ CONFIG.urlmap_basename }}"></script>
  {% endif %}
  {% if CONFIG.search_index %}
  <script src="fzy.js"></script>
  <script src="search.js"></script>
  {% endif %}
  <script src="main.js"></script>

  {% for f in CONFIG.favicons %}
    {% if f.endswith("180x180.png") %}
  <link rel="apple-touch-icon" sizes="180x180" href="{{ f }}">
    {% elif f.endswith("192x192.png") %}
  <link rel="icon" sizes="192x192" href="{{ f }}">
    {% elif f.endswith("128x128.png") %}
  <link rel="icon" sizes="128x128" href="{{ f }}">
    {% elif f.endswith("32x32.png") %}
  <link rel="icon" sizes="32x32" href="{{ f }}">
    {% else %}
  <link rel="icon" href="{{ f }}">
    {% endif %}
  {% endfor %}
</head>

<body>
  <div id="body-wrapper" tabindex="-1">

    <nav class="sidebar devhelp-hidden">
      {% if CONFIG.logo_url %}
      <div class="section">
        <a href="index.html"><img src="{{ CONFIG.logo_url }}" class="logo"/></a>
      </div>
      {% endif %}
      {% if CONFIG.search_index %}
      <div class="search section">
        <form id="search-form" autocomplete="off">
          <input id="search-input" type="text" name="do-not-autocomplete" placeholder="Click, or press 's' to search" autocomplete="off"/>
        </form>
      </div>
      {% endif %}
      <div class="section namespace">
        <h3><a href="index.html">{{ namespace.name }}</a></h3>
        <p>API Version: {{ namespace.version }}</p>
        {% if CONFIG.version %}
        <p>Library Version: {{ CONFIG.version }}</p>
        {% endif %}
      </div>
      {% block sidebar %}
      {% endblock %}
      <div class="section generator">
        <p>Generated by <a href="https://gitlab.gnome.org/GNOME/gi-docgen">gi-docgen</a> {{ CONFIG.generator }}</p>
      </div>
    </nav>

    <button id="btn-to-top" class="hidden"><span class="up-arrow"></span></button>

    {% block content %}{% endblock %}

    {% block navbar %}{% endblock %}

    <section id="search" class="content hidden"></section>

    <footer>
    {% block footer %}{% endblock %}
    </footer>
  </div>
</body>
</html>
